/*
   Copyright 2006-2009 Murali Krishnan

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
 */

package com.notuvy.clip;

import java.awt.Toolkit;
import java.awt.datatransfer.*;
import java.io.*;
import java.util.*;
import java.util.regex.*;

import org.apache.log4j.Logger;

/**
 * A utility that provides read/write access to the system Clipboard.
 *
 * @author  murali
 */
public class ClipboardAccess implements ClipboardOwner {

    //------------------------------------------------------------
    //- Class Variables

    protected static final Logger LOG = Logger.getLogger(ClipboardAccess.class);

    private static ClipboardAccess sInstance = new ClipboardAccess();


    //------------------------------------------------------------
    //- Class Functions

    public static ClipboardAccess getInstance() {
        return sInstance;
    }


    public static String read() {
        String result = getInstance().readText();
        return result;
    }


    public static void save(String pString) {
        getInstance().saveText(pString);
    }


    //------------------------------------------------------------
    //- Instance Variables

    private final Clipboard vClipboard;


    //------------------------------------------------------------
    //- Constructors

    private ClipboardAccess() {
        vClipboard = Toolkit.getDefaultToolkit().getSystemClipboard();
    }


    //------------------------------------------------------------
    //- Accessors

    protected Clipboard getClipboard() {
        return vClipboard;
    }


    //------------------------------------------------------------
    //- Settors


    //------------------------------------------------------------
    //- Private/Protected Utility Functions

    public String readText() {
        String result = "";

        //odd: the Object param of getContents is not currently used
        Transferable contents = getClipboard().getContents(this);
        boolean hasTransferableText =
            (contents != null) &&
            contents.isDataFlavorSupported(DataFlavor.stringFlavor);
        if ( hasTransferableText ) {
            try {
                result = (String)contents.getTransferData(DataFlavor.stringFlavor);
                if (result == null) {
                    result = "";
                }
            } catch (UnsupportedFlavorException ex){
                //highly unlikely since we are using a standard DataFlavor
                LOG.error("Unsupported", ex);
            } catch (IOException ex) {
                LOG.debug("Basic problem", ex);
            }
        }

        return result;
    }


    public void saveText(String pString) {
        StringSelection selection = new StringSelection(pString);
        getClipboard().setContents(selection, this);
        ClipboardCallback.interrupt();
    }


    //------------------------------------------------------------
    //- Public Interface Functions



    //------------------------------------------------------------
    //- Class Interface Functions

    public void lostOwnership(Clipboard pClipboard, Transferable pTransferable) {
    }


    //------------------------------------------------------------
    //- Inner Classes



    //------------------------------------------------------------
    //- Main

    public static void main(String[] args) {
        try {
            ClipboardAccess clipboard = new ClipboardAccess();
            clipboard.read();
        } catch (Throwable thr) {
            LOG.error("Fatal error.", thr);
        }
    }


}
